#include <htc.h>
#include "init.h"
#include "lcd.h"

#ifndef _XTAL_FREQ
// Unless specified elsewhere, 4MHz system frequency is assumed
#define _XTAL_FREQ 4000000
#endif

#define E_TRIS TRISA5 // TRIS for E pin
#define E RA5 // LCD Enable pin
#define LCD_DATA_TRIS TRISC // TRIS for data port
#define LCD_DATA PORTC // LCD data port

// Macro for LCD Enable
#define LCD_STROBE() ((E=1), (E=0))

void putch(unsigned char c) {
  // Load only high 4 bits of character with 'RS' bit to 1
  LCD_DATA=((c>>4)&0x0F|0b00100000);
  LCD_STROBE();
  // Load only low 4 bits of character with 'RS' bit to 1
  LCD_DATA=(c&0x0F|0b00100000);
  LCD_STROBE();

  // Wait for more than 43us
  __delay_us(46); // Wait for 46us
}

/*
 * Sends a command to LCD without wait.
 */
void lcd_command(unsigned char c) {
  // Load only high 4 bits of character
  LCD_DATA=((c>>4)&0x0F);
  LCD_STROBE();
  // Load only low 4 bits of character
  LCD_DATA=(c&0x0F);
  LCD_STROBE();
}

/*
 * Function set. Sets LCD to 4-bit, 1-line mode, 5x7 dots.
 */
void lcd_functionSet(void) {
  // Additional set for 4-bit mode
  LCD_DATA=0b00000010;
  LCD_STROBE();

  lcd_command(0b00101000);

  // Wait for more than 39us
  __delay_us(42); // wait for 42us
}

/*
 * Display on/off control.
 */
void lcd_displayMode(void) {
  lcd_command(0b00001100); // no cursor

  // Wait for more than 39us
  __delay_us(42); // wait for 42us
}

void lcd_clearDisplay(void) {
  lcd_command(0b00000001);

  // Wait for more than 1.53ms
  __delay_ms(2); // wait for 2ms
}

/*
 * Entry mode set.
 */
void lcd_entrySet(void) {
  lcd_command(0b00000110);

  // Wait for more than 39us
  __delay_us(42); // wait for 42us
}

void lcd_returnHome(void) {
  lcd_command(0b00000010);

  // Wait for more than 1.53ms
  __delay_ms(2); // wait for 2ms
}

void lcd_nextLine(void) {
  lcd_command(0b11000000);

  // Wait for more than 1.53ms
  __delay_ms(2); // wait for 2ms
}

void lcd_setup(void) {
  E_TRIS=0; // set as output
  LCD_DATA_TRIS=0; // set as output

  E=0;
  LCD_DATA=0;

  // Wait for more than 30ms after VDD rises to 4.5V
  __delay_ms(197); // wait max, 197ms

  // Function set
  lcd_functionSet();

  // Display on/off control
  lcd_displayMode();

  // Clear display
  lcd_clearDisplay();

  // Entry mode set
  lcd_entrySet();
}
